library vectorx;

import 'dart:convert';
import 'package:vectorx/vectorx/basic/align_widget_parser.dart';
import 'package:vectorx/vectorx/basic/button_widget_parser.dart';
import 'package:vectorx/vectorx/basic/container_widget_parser.dart';
import 'package:vectorx/vectorx/basic/text_widget_parser.dart';
import 'package:vectorx/vectorx/scrolling/pageview_widget_parser.dart';
import 'package:flutter/widgets.dart';
import 'package:logging/logging.dart';

class VectorXBuilder {
  static final Logger log = Logger('VectorX');

  static final _parsers = [
    ContainerWidgetParser(),
    AlignWidgetParser(),
    PageViewWidgetParser(),
    RaisedButtonParser(),
    TextWidgetParser(),
  ];

  static final _widgetNameParserMap = <String, WidgetParser>{};

  static bool _defaultParserInited = false;

  // use this method for adding your custom widget parser
  static void addParser(WidgetParser parser) {
    log.info(
        "add custom widget parser, make sure you don't overwirte the widget type.");
    _parsers.add(parser);
    _widgetNameParserMap[parser.widgetName] = parser;
  }

  static void initDefaultParsersIfNess() {
    if (!_defaultParserInited) {
      for (var parser in _parsers) {
        _widgetNameParserMap[parser.widgetName] = parser;
      }
      _defaultParserInited = true;
    }
  }

  static Widget build(String json, BuildContext buildContext,
      ClickListener listener, Map options) {
    initDefaultParsersIfNess();
    var map = jsonDecode(json);
    ClickListener _listener =
        listener == null ? new NonResponseWidgetClickListener() : listener;
    var widget = buildFromMap(map, buildContext, _listener);
    return widget;
  }

  static Widget buildFromMap(Map<String, dynamic> map,
      BuildContext buildContext, ClickListener listener) {
    initDefaultParsersIfNess();
    if (map == null) {
      return null;
    }
    String widgetName = map['type'];
    if (widgetName == null) {
      return null;
    }
    var parser = _widgetNameParserMap[widgetName];
    if (parser != null) {
      return parser.parse(map, buildContext, listener);
    }
    log.warning("Not support parser type: $widgetName");
    return null;
  }

  static List<Widget> buildWidgets(
      List<dynamic> values, BuildContext buildContext, ClickListener listener) {
    initDefaultParsersIfNess();
    List<Widget> rt = [];
    for (var value in values) {
      rt.add(buildFromMap(value, buildContext, listener));
    }
    return rt;
  }

  static Map<String, dynamic> export(Widget widget, BuildContext buildContext) {
    initDefaultParsersIfNess();
    var parser = _findMatchedWidgetParserForExport(widget);
    if (parser != null) {
      return parser.export(widget, buildContext);
    }
    log.warning(
        "Can't find WidgetParser for Type ${widget.runtimeType} to export.");
    Map<String, dynamic> t = {};
    return t;
  }

  static List<Map<String, dynamic>> exportWidgets(
      List<Widget> widgets, BuildContext buildContext) {
    initDefaultParsersIfNess();
    List<Map<String, dynamic>> rt = [];
    for (var widget in widgets) {
      rt.add(export(widget, buildContext));
    }
    return rt;
  }

  static WidgetParser _findMatchedWidgetParserForExport(Widget widget) {
    for (var parser in _parsers) {
      if (parser.matchWidgetForExport(widget)) {
        return parser;
      }
    }
    return null;
  }
}

/// extends this class to make a Flutter widget parser.
abstract class WidgetParser {
  /// parse the json map into a flutter widget.
  Widget parse(Map<String, dynamic> map, BuildContext buildContext,
      ClickListener listener);

  /// the widget type name for example:
  /// {"type" : "Text", "data" : "Denny"}
  /// if you want to make a flutter Text widget, you should implement this
  /// method return "Text", for more details, please see
  /// @TextWidgetParser
  String get widgetName;

  /// export the runtime widget to json
  Map<String, dynamic> export(Widget widget, BuildContext buildContext);

  /// match current widget
  Type get widgetType;

  bool matchWidgetForExport(Widget widget) => widget.runtimeType == widgetType;
}

abstract class ClickListener {
  void onClicked(String event);
}

class NonResponseWidgetClickListener implements ClickListener {
  static final Logger log = Logger('NonResponseWidgetClickListener');

  @override
  void onClicked(String event) {
    log.info("receiver click event: " + event);
    print("receiver click event: " + event);
  }
}
